__title__ = 'Meraki_Auto_Sync'
__description__ = 'Auto Sync Meraki Network Dashboard cross Orginization based on network tags for master and target'
__url__ = 'http://devnet.cisco.com'
__download_url__ = 'https://devnet.cisco.com'
__author__ = 'Josh Lipton'
__author_email__ = 'jolipton@cisco.com'
__copyright__ = "Copyright (c) 2021 Cisco Systems, Inc."
__license__ = "MIT"
